// ExperSwarm.m

#import "ExperSwarm.h"
#import "ModelSwarm.h"
#import <collections.h>
#import <objectbase.h>
#import <analysis.h>

@implementation ExperSwarm

+createBegin: (id) aZone {
  ExperSwarm * obj;

  obj = [super createBegin: aZone];

  obj->parameterName      =strdup("p1");
  obj->lowerBoundParameter=0.1;
  obj->upperBoundParameter=0.9;
  obj->incrementParameter =0.1;
  obj->numGenerations     =1000;
  obj->lowerBoundSeed     =1;
  obj->upperBoundSeed     =20;
  obj->plotCodeKnowledge  =NO;
   
  return obj;
}

-createEnd {
  [ObjectLoader load: self fromFileNamed: "exper.setup"];
  return [super createEnd];
}

-buildObjects {
  [super buildObjects];
  
  [probeDisplayManager createProbeDisplayFor: self];
  [controlPanel setStateStopped];
  
   graph = [Graph createBegin: self];
   graph = [graph createEnd];
   [graph setTitle: parameterName];
   [graph setWidth: 400 Height: 300];

   knowledge = [graph createElement];
   [knowledge setLabel: "knowledge"];

  return self;
}


-activateIn: (id) swarmContext {
  [super activateIn: swarmContext];
  return [self getSwarmActivity];
}

-(double)getKnowledge {
  if (plotCodeKnowledge)
    return [modelSwarm getCodeKnowledge];
  else
    return [modelSwarm getAverageKnowledge];
}

-run {
  double x,y;
  double value;
  double parameter;
  int numRepl = 0;
  int seed;
  int g, i;
  id <OutFile> outFile;
  id <OutFile> dataFile;
  char string[100];
  BOOL done;
  double avKnowledge, kw;
  
  
  dataFile=[OutFile create: self withName: parameterName];
  
  for(parameter=lowerBoundParameter;
      parameter<=upperBoundParameter;
      parameter+=incrementParameter) {
    numRepl = 0;
    avKnowledge = 0.0;

    printf("\n");
    printf("Parameter: %6.2f ", parameter);
    for(seed=lowerBoundSeed;seed<=upperBoundSeed;seed++) {
      numRepl++;

      // Create setup file for ModelSwarm
      sprintf(string,"@begin\nrandomSeed %d\n%s %f\n@end\n",seed,parameterName,parameter);
      
      outFile=[OutFile create: self withName: "loop.setup"];
      [outFile putString: string];
      [outFile drop];
      
      // Run ModelSwarm
      modelSwarm = [ModelSwarm create: self];
      [ObjectLoader load: modelSwarm fromFileNamed: "loop.setup"];
      [modelSwarm buildObjects];

      // Run the model and get results

      g = 0;
      done = NO;

      while ((g<numGenerations) && !done) {
	[modelSwarm step];
	if ((g % 10 == 0) && (g > 19))
	  if ([[modelSwarm getOrg] isEquil])
	    done = YES;
	g++;
      }

      if (plotCodeKnowledge)
	kw = [[modelSwarm getOrg] getCodeKnowledge];
      else
	kw = [[modelSwarm getOrg] getAverageKnowledge];

      avKnowledge = avKnowledge + kw;

      [modelSwarm drop];
    
      [dataFile putLong: seed];
      [dataFile putTab];
      [dataFile putDouble: parameter];
      [dataFile putTab];
      [dataFile putDouble: kw];
      [dataFile putNewLine];
      
    }
    avKnowledge = avKnowledge / (double)numRepl;
    [knowledge addX: parameter Y: avKnowledge];
    printf("Average = %6.2f \n", avKnowledge);
  }
  [graph pack];
  [controlPanel setStateStopped];

  return self;
}




@end








