// ModelSwarm.m				

#import "ModelSwarm.h"
#import <simtools.h>

@implementation ModelSwarm

+createBegin: (id)  aZone {
  ModelSwarm * obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];

  obj->randomSeed = 1;
  obj->numFeatures = 30;
  obj->numMembers = 50;
  obj->p1 = 0.3;
  obj->p2 = 0.9;
  obj->p3 = 0.0;
  obj->p4 = 0.0;
  obj->printFlag = NO;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "randomSeed"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numMembers"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numFeatures"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "p1"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "p2"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "p3"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "p4"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "printFlag"
				    inClass: [self class]]];
  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

-createEnd {
  [ObjectLoader load: self fromFileNamed: "model.setup"];
  return [super createEnd];
}

-buildObjects {

  [randomGenerator setStateFromSeed: randomSeed];

  time = 0;

  envir = [Environment create: self];
  [envir initFeatures: numFeatures];

  org = [Organization create: self];
  [org initMembers: numMembers Features: numFeatures Reality: [envir getReality]];
    
  return self;
}

-(id) getMembers {
  return [org getMembers];
}

-(id) getOrg {
  return org;
}

-(id) getEnvir {
  return envir;
}

-reportResults {
  printf("T = %d\n", time);
  printf("Real: ");
  [[envir getReality] print]; 
  printf("\n");
  [org print];
  
  return self;
}

-step {
  if ((time==0) && printFlag)
    [self reportResults];

  [org socialize: p1];
  [org adaptCode: p2];
  [org updateBeliefs];
  if (p3 > 0.0)
    [org turnover: p3];
  if (p4 > 0.0)
    [envir turbulence: p4];

  if (printFlag)
    [self reportResults];

  time++;

  return self;
}

@end




































































  


