// ObserverSwarm.m

#import "ObserverSwarm.h"
#import "ModelSwarm.h"
#import <collections.h>
#import <objectbase.h>
#import <analysis.h>

@implementation ObserverSwarm

+createBegin: (id) aZone {
  ObserverSwarm * obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];
  obj->displayFrequency = 1;
  obj->plotCodeKnowledge = NO;
  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];  
  [probeMap addProbe: [probeLibrary getProbeForVariable: "plotCodeKnowledge"
				    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

-createEnd {
  return [super createEnd];
}

-buildObjects {
  [super buildObjects];
  modelSwarm = [ModelSwarm create: self];
  [probeDisplayManager createProbeDisplayFor: modelSwarm];  
  [probeDisplayManager createProbeDisplayFor: self];

  [controlPanel setStateStopped];
  [modelSwarm buildObjects];

  knowledgeGraph = [EZGraph createBegin: self];
  if (plotCodeKnowledge)
    [knowledgeGraph setTitle: "Code Knowledge"];
  else
    [knowledgeGraph setTitle: "Average Knowledge"];
  [knowledgeGraph setAxisLabelsX: "Time" Y: "Knowledge Level"];
  knowledgeGraph = [knowledgeGraph createEnd];

  if (plotCodeKnowledge)
    [knowledgeGraph createSequence: "Knowledge" 
		    withFeedFrom: [modelSwarm getOrg] 
		    andSelector: M(getCodeKnowledge)];
  else
    [knowledgeGraph createSequence: "Knowledge" 
		    withFeedFrom: [modelSwarm getOrg] 
		    andSelector: M(getAverageKnowledge)];

  
  [controlPanel setStateStopped];


  return self;
}

-buildActions {

  [super buildActions];

  displayActions = [ActionGroup create: self];
  [displayActions createActionTo: modelSwarm         message: M(step)];
  [displayActions createActionTo: knowledgeGraph      message: M(step)];
  [displayActions createActionTo: actionCache        message: M(doTkEvents)];

  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: 1]; // note frequency!
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];
 
  return self;
}

-activateIn: (id) swarmContext {
  [super activateIn: swarmContext];
  [displaySchedule activateIn: self];
  return [self getSwarmActivity];
}

@end








