// Organization.m				
  
#import "Organization.h"
#import <simtools.h>

@implementation Organization 

-initMembers: (int) n Features: (int) nf Reality: (id) r {
  int i;
  id aMember;

  numMembers = n;
  numFeatures = nf;
  reality = r;

  code = [Culture create: [self getZone]];

  [code initFeatures: nf Traits: 0];

  members = [Array create: [self getZone]  setCount: numMembers];

  for (i = 0; i < numMembers; i++) {
    aMember = [OrgMember create: [self getZone]];
    [aMember init: i Features: numFeatures];
    [members atOffset: i put: aMember];
  }
  
  return self;
}

-(id)getCode {
  return code;
}

-(int)fit: (id) b {
  int s=0;
  int j;

  for (j=0; j<numFeatures; j++)
    if ([b getFeature: j] == 0)
      ;
    else if ([b getFeature: j] == [reality getFeature: j])
      s++;
    else
      s--; 
 
  return s;
}

-(double)getCodeKnowledge {
  return (double)[self fit: code] / (double)numFeatures;
}

-(double)getAverageKnowledge {
  long int s=0;
  int i;

  for (i=0; i<numMembers; i++)
    s = s + [self fit: [[members atOffset: i] getBelief]];
  
  return (double)s / (double)numMembers / (double)numFeatures;
}

-socialize: (double) p1 {
  int i;

  for (i=0; i<numMembers; i++)
    [[members atOffset: i] adapt: p1 to: code];
  return self;
}

-(BOOL)isEquil {
  int i, j;
  BOOL same=YES;

  i = 0;
  while ((i<numMembers) && same) {
    j = 0;
    while ((j<numFeatures) && same) {
      if ([[[members atOffset: i] getBelief] getFeature: j] != [code getFeature: j])
	same = NO;
      j++;
    }
    i++;
  }
  return same;
}

-(int)calcSup {
  int i, x;
  int n=0;

  x = [self fit: code];
  for (i=0; i<numMembers; i++)
    if ([self fit: [[members atOffset: i] getBelief]] > x) {
      sup[n]=i;
      n++;
    }
  return n;
}

-(int)calcK: (int) n Dim: (int) j {
  int i;
  int k=0;

  for (i=0; i<n; i++)
    if ([[[members atOffset: sup[i]] getBeliefStar] getFeature: j] == 1)
      k++;
    else
      k--;

  return k;
}

-adaptCode: (double) p2 {
  int i,j,k,l,numSup;
  BOOL done;

  for (i=0; i<numMembers; i++)
    [[members atOffset: i] completeBelief];
  numSup = [self calcSup];
  
  for (j=0; j<numFeatures; j++) {
    k=[self calcK: numSup Dim: j];
    if (k*[code getFeature: j] <= 0) {
      done = NO;
      l=abs(k);
      while ((l>0) && !done) {
	if ([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0] < p2) {
	  if (k>0)
	    [code setFeature: j to: 1];
	  else
	    [code setFeature: j to: -1];
	  done = YES;
	}
	l = l - 1;
      }
    }
  }
  return self;
}

-updateBeliefs {
  int i;
  
  for (i=0; i<numMembers; i++)
    [[members atOffset: i] updateBelief];
  return self;
}

-turnover: (double) p3 {
  int i;

  for (i=0; i<numMembers; i++)
    if ([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0] < p3)
      [[[members atOffset: i] getBelief] randomize: 3];	
  return self;
}

-print {
  int i;

  printf("Code: "); 
  [code print]; 
  printf("\n");
  for (i=0; i<numMembers; i++) {
    printf("  %2d: ",i); 
    [[[members atOffset: i] getBelief] print];
    printf("\n");
  }
  printf("\n");
  return self;
}

@end

  


